﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using Verse;

namespace TROM
{
    class SketchResolver_HouseEdge_R11_1 : SketchResolver
    {
        public static int x = 11, z = 11;

        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            ThingDef wallMaterial = Utility.Filters.RandStone();
            bool backhouse = (bool)parms.monumentOpen;//is backhouse?
            bool backhouseSide = Rand.Bool;//left(false) or right(true)?

            int width = x;
            int height = 7;
            Sketch house = new Sketch();
            bool onlyBuildableByPlayer = parms.onlyBuildableByPlayer ?? false;
            bool filterAllowsAll = parms.allowedMonumentThings == null;

            //wall array
            bool[,] array1 = Utility.BoxShapeGenerator.Generate(5, 7);
            bool[,] array = new bool[11, 11];
            for (int i = 0; i < array1.GetLength(0); i++)
            {
                for (int j = 0; j < array1.GetLength(1); j++)
                {
                    if (array1[i, j])
                    {
                        array[i, j] = true;
                        array[i + 6, j] = true;
                    }
                }
            }
            //backhouse walls
            if (backhouse)
            {
                if (backhouseSide)
                {
                    array[2, 8] = true;
                    array[2, 9] = true;
                    array[2, 10] = true;
                    array[3, 10] = true;
                    array[4, 10] = true;
                    array[5, 10] = true;
                    array[6, 10] = true;
                    array[7, 10] = true;
                    array[8, 10] = true;
                    array[9, 10] = true;
                    array[10, 10] = true;
                    array[10, 9] = true;
                    array[10, 8] = true;
                    array[10, 7] = true;
                }
                else
                {
                    array[0, 7] = true;
                    array[0, 8] = true;
                    array[0, 9] = true;
                    array[0, 10] = true;
                    array[1, 10] = true;
                    array[2, 10] = true;
                    array[3, 10] = true;
                    array[4, 10] = true;
                    array[5, 10] = true;
                    array[6, 10] = true;
                    array[7, 10] = true;
                    array[8, 10] = true;
                    array[8, 8] = true;
                    array[8, 9] = true;
                }
            }

            //space for doors
            array[4, 3] = false;
            array[6, 3] = false;


            //add walls
            for (int i = 0; i < array.GetLength(0); i++)
            {
                for (int j = 0; j < array.GetLength(1); j++)
                {
                    if (array[i, j])
                    {
                        house.AddThing(ThingDefOf.Wall, new IntVec3(i, 0, j), Rot4.North, wallMaterial);
                    }
                }
            }

            ResolveParams parms2 = parms;
            parms2.sketch = house;
            parms2.connectedGroupsSameStuff = true;
            if (parms.addFloors ?? true)
            {
                ResolveParams parms3 = parms;
                parms3.thingCentral = ThingDefOf.WoodLog;
                parms3.singleFloorType = true;
                parms3.sketch = house;
                parms3.floorFillRoomsOnly = false;
                parms3.onlyStoneFloors = parms.onlyStoneFloors ?? false;
                parms3.allowConcrete = parms.allowConcrete ?? false;
                //bedroom floors
                parms3.rect = new CellRect(1, 1, 3, 5);
                TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                parms3.rect = new CellRect(7, 1, 3, 5);
                TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                if (backhouse)
                {
                    if (backhouseSide)
                    {
                        parms3.rect = new CellRect(3, 7, 7, 3);
                    }
                    else
                    {
                        parms3.rect = new CellRect(1, 7, 7, 3);
                    }
                    TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                }
                //hallway floor
                parms3.rect = new CellRect(4, 0, 3, 7);
                parms3.thingCentral = wallMaterial;
                TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                if (backhouse)
                {
                    if (backhouseSide)
                    {
                        parms3.rect = new CellRect(2, 7, 1, 1);
                    }
                    else
                    {
                        parms3.rect = new CellRect(8, 7, 1, 1);
                    }
                    parms3.thingCentral = wallMaterial;
                    TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                }
            }
            List<SketchThing> things = house.Things;
            for (int num6 = 0; num6 < things.Count; num6++)
            {
                if (things[num6].def == ThingDefOf.Wall)
                {
                    house.RemoveTerrain(things[num6].pos);
                }
            }

            //add doors
            house.AddThing(ThingDefOf.Door, new IntVec3(4, 0, 3), Rot4.East);
            house.AddThing(ThingDefOf.Door, new IntVec3(6, 0, 3), Rot4.East);
            house.AddThing(ThingDefOf.Door, new IntVec3(5, 0, 1), Rot4.North);
            house.AddThing(ThingDefOf.Door, new IntVec3(5, 0, 6), Rot4.North);
            if (backhouse)
            {
                if (backhouseSide)
                {
                    house.AddThing(ThingDefOf.Door, new IntVec3(2, 0, 7), Rot4.East);
                }
                else
                {
                    house.AddThing(ThingDefOf.Door, new IntVec3(8, 0, 7), Rot4.East);
                }
            }

            //debris
            if (backhouse)
            {
                if (Rand.Bool) house.AddThing(ThingDefOf.AncientBed, new IntVec3(Rand.Range(1, 4), 0, 5), Rot4.South);
                if (Rand.Bool) house.AddThing(ThingDefOf.AncientBed, new IntVec3(Rand.Range(7, 10), 0, 5), Rot4.South);

                if (backhouseSide)
                {
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientKitchenSink, new IntVec3(1, 0, 8), Rot4.East);
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientWashingMachine, new IntVec3(5, 0, 9), Rot4.North);
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientToilet, new IntVec3(7, 0, 9), Rot4.North);
                }
                else
                {
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientKitchenSink, new IntVec3(3, 0, 8), Rot4.West);
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientWashingMachine, new IntVec3(3, 0, 9), Rot4.North);
                    if (Rand.Bool) house.AddThing(TROMThingDefOf.AncientToilet, new IntVec3(5, 0, 9), Rot4.North);
                }
            }

            parms.sketch.MergeAt(house, new IntVec3(parms.rect.Value.minX, 0, parms.rect.Value.minZ), Sketch.SpawnPosType.Unchanged);
        }
    }
}
